import { FontManager, FONT_FAMILY_DEFAULT, OPTIONS_DEFAULTS, getFontId } from '@samuelmeuli/font-manager';

class FontPicker {
    constructor(apiKey, defaultFamily = FONT_FAMILY_DEFAULT, { pickerId = OPTIONS_DEFAULTS.pickerId, families = OPTIONS_DEFAULTS.families, categories = OPTIONS_DEFAULTS.categories, scripts = OPTIONS_DEFAULTS.scripts, variants = OPTIONS_DEFAULTS.variants, limit = OPTIONS_DEFAULTS.limit, sort = OPTIONS_DEFAULTS.sort, }, onChange = () => { }) {
        this.expanded = false;
        this.closeEventListener = this.closeEventListener.bind(this);
        this.toggleExpanded = this.toggleExpanded.bind(this);
        const options = {
            pickerId,
            families,
            categories,
            scripts,
            variants,
            limit,
            sort,
        };
        this.fontManager = new FontManager(apiKey, defaultFamily, options, onChange);
        this.generateUI(sort);
    }
    generateUI(sort) {
        const { selectorSuffix } = this.fontManager;
        const pickerId = `font-picker${selectorSuffix}`;
        this.fontPickerDiv = document.getElementById(pickerId);
        if (!this.fontPickerDiv) {
            throw Error(`Missing div with id="${pickerId}"`);
        }
        const dropdownButton = document.createElement("button");
        dropdownButton.classList.add("dropdown-button");
        dropdownButton.onclick = this.toggleExpanded;
        dropdownButton.onkeypress = this.toggleExpanded;
        dropdownButton.type = "button";
        this.fontPickerDiv.appendChild(dropdownButton);
        this.dropdownFamily = document.createElement("p");
        this.dropdownFamily.textContent = this.fontManager.getActiveFont().family;
        this.dropdownFamily.classList.add("dropdown-font-family");
        dropdownButton.appendChild(this.dropdownFamily);
        const dropdownIcon = document.createElement("p");
        dropdownIcon.classList.add("dropdown-icon", "loading");
        dropdownButton.appendChild(dropdownIcon);
        this.fontManager
            .init()
            .then((fontMap) => {
            const fonts = Array.from(fontMap.values());
            if (sort === "alphabet") {
                fonts.sort((font1, font2) => font1.family.localeCompare(font2.family));
            }
            this.generateFontList(fonts);
            dropdownIcon.classList.replace("loading", "finished");
        })
            .catch((err) => {
            dropdownIcon.classList.replace("loading", "error");
            console.error("Error trying to fetch the list of available fonts");
            console.error(err);
        });
    }
    generateFontList(fonts) {
        this.ul = document.createElement("ul");
        this.ul.classList.add("font-list");
        fonts.forEach((font) => {
            this.addFontLi(font);
        });
        this.fontPickerDiv.appendChild(this.ul);
        const activeFontFamily = this.fontManager.getActiveFont().family;
        const activeFontId = getFontId(activeFontFamily);
        const fontButtonId = `font-button-${activeFontId}${this.fontManager.selectorSuffix}`;
        this.activeFontButton = document.getElementById(fontButtonId);
        if (this.activeFontButton) {
            this.activeFontButton.classList.add("active-font");
        }
        else {
            console.error(`Could not find font button with ID (${fontButtonId})`);
        }
    }
    addFontLi(font, listIndex) {
        const fontId = getFontId(font.family);
        const li = document.createElement("li");
        li.classList.add("font-list-item");
        const fontButton = document.createElement("button");
        fontButton.type = "button";
        fontButton.id = `font-button-${fontId}${this.fontManager.selectorSuffix}`;
        fontButton.classList.add("font-button");
        fontButton.textContent = font.family;
        const onActivate = () => {
            this.toggleExpanded();
            this.setActiveFont(font.family);
        };
        fontButton.onclick = onActivate;
        fontButton.onkeypress = onActivate;
        li.appendChild(fontButton);
        if (listIndex) {
            this.ul.insertBefore(li, this.ul.children[listIndex]);
        }
        else {
            this.ul.appendChild(li);
        }
    }
    closeEventListener(e) {
        let targetEl = e.target;
        const fontPickerEl = document.getElementById(`font-picker${this.fontManager.selectorSuffix}`);
        while (true) {
            if (targetEl === fontPickerEl) {
                return;
            }
            if (targetEl.parentNode) {
                targetEl = targetEl.parentNode;
            }
            else {
                this.toggleExpanded();
                return;
            }
        }
    }
    toggleExpanded() {
        if (this.expanded) {
            this.expanded = false;
            this.fontPickerDiv.classList.remove("expanded");
            document.removeEventListener("click", this.closeEventListener);
        }
        else {
            this.expanded = true;
            this.fontPickerDiv.classList.add("expanded");
            document.addEventListener("click", this.closeEventListener);
        }
    }
    getFonts() {
        return this.fontManager.getFonts();
    }
    addFont(fontFamily, index) {
        if (Array.from(this.fontManager.getFonts().keys()).includes(fontFamily)) {
            throw Error(`Did not add font to font picker: Font family "${fontFamily}" is already in the list`);
        }
        this.fontManager.addFont(fontFamily, true);
        const font = this.fontManager.getFonts().get(fontFamily);
        if (font) {
            this.addFontLi(font, index);
        }
        else {
            console.error(`Font "${fontFamily}" is missing in font list`);
        }
    }
    removeFont(fontFamily) {
        this.fontManager.removeFont(fontFamily);
        const fontId = getFontId(fontFamily);
        const fontButton = document.getElementById(`font-button-${fontId}${this.fontManager.selectorSuffix}`);
        if (fontButton) {
            const fontLi = fontButton.parentElement;
            fontButton.remove();
            if (fontLi) {
                fontLi.remove();
            }
        }
        else {
            throw Error(`Could not remove font from font picker: Font family "${fontFamily}" is not in the list`);
        }
    }
    getActiveFont() {
        return this.fontManager.getActiveFont();
    }
    setActiveFont(fontFamily) {
        this.fontManager.setActiveFont(fontFamily);
        const fontId = getFontId(fontFamily);
        this.dropdownFamily.textContent = fontFamily;
        if (this.activeFontButton) {
            this.activeFontButton.classList.remove("active-font");
            this.activeFontButton = document.getElementById(`font-button-${fontId}${this.fontManager.selectorSuffix}`);
            this.activeFontButton.classList.add("active-font");
        }
        else {
            console.error("`activeFontButton` is undefined");
        }
    }
    setOnChange(onChange) {
        this.fontManager.setOnChange(onChange);
    }
}

export default FontPicker;
